# netports

A collection of Python tools for managing ranges of vlans, tcp/udp ports, interfaces. Useful for
telecom network related scripting.

# Installation

```bash
pip install netports
```

# Code Documentation

## netports.ranges(line, splitter, range_splitter)

**Ranges of digits**. Convert `line` to _object_ witch contain range as _str_ and as list of _int_.
Parameter `line` can be unsorted and with duplicates. Result will be ordered and without duplicates.

    Parameters      Description
    ==============  ================================================================================
    line            Range in <str> format.
    splitter        Delimiter between items. By default ",".
    range_splitter  Delimiter between min and max digits in range. By default "-".

```python
import netports

# Convert <str> to <object>, remove duplicates and sort ranges. Cisco style.
assert netports.Ranges("3-5,1,3-5,1") == netports.Ranges("1,3-5")
ranges = netports.Ranges("1,3-5")
print(ranges.line)
print(ranges.ports)
# 1,3-5
# [1, 3, 4, 5]

# Change default splitter and range_splitter
ranges = netports.Ranges("1 3 to 5", splitter=" ", range_splitter=" to ")
print(ranges.line)
print(ranges.ports)
# 1 3 to 5
# [1, 3, 4, 5]
```

## netports.iports(items)

**Integer Ports**. Convert `items` to list of _int_. Parameter `items` can be as _str_, as list of _
str_, as list of _int_. Parameter `items` can be unsorted and with duplicates. Output data will be
ordered and without duplicates.

    Parameters      Description
    ==============  ================================================================================
    line            Range in <str> format.
    splitter        Delimiter between items. By default ",".
    range_splitter  Delimiter between min and max digits in range. By default "-".

```python
import netports

# Convert <str> to list of <int>. Output ordered and without duplicates.
assert netports.iports("1,3-5") == [1, 3, 4, 5]
assert netports.iports("1 , 3 - 5, 1, 3-4, 4-5\n") == [1, 3, 4, 5]
# Convert list of <str> to list of <int>. Output ordered and without duplicates.
assert netports.iports(["1,7-9", "3-5"]) == [1, 3, 4, 5, 7, 8, 9]
# Convert list <int> to list of <int>. Output ordered and without duplicates.
assert netports.iports([5, 5, 1, 3, 4]) == [1, 3, 4, 5]
```

## netports.sports(items)

**String Ports**. Convert `items` to _str_. Parameter `items` can be as _str_, as list of
_str_, as list of _int_. Parameter `items` can be unsorted and with duplicates. Output will be
ordered and without duplicates.

```python
import netports

# Convert <str> to sorted <str>. Output ordered and without duplicates.
assert netports.sports("1,3-5") == "1,3-5"
assert netports.sports("1 , 3 - 5, 1, 3-4, 4-5\n") == "1,3-5"
# Convert list of <str> to sorted <str>. Output ordered and without duplicates.
assert netports.sports(["1,7-9", "3-5"]) == "1,3-5,7-9"
# Convert list <int> to sorted <str>. Output ordered and without duplicates.
assert netports.sports([5, 5, 1, 3, 4]) == "1,3-5"
```

## netports.itcp(items)

**Integer TCP ports**. Convert `items` to list of _int_. Parameter `items` can be as _str_, as list
of _str_, as list of _int_. Parameter `items` can be unsorted and with duplicates. Output data will
be ordered and without duplicates. Raise ValueError if ports are outside valid TCP/UDP ports
1...65535.

```python
import netports

# Convert <str> to list of <int>. Output ordered and without duplicates.
assert netports.itcp("1,3-5,1,3-5") == [1, 3, 4, 5]
# Raise ValueError if items are outside of range 1...65535.
try:
    netports.itcp("65536")
except ValueError:
    print("invalid tcp port")
```

## netports.stcp(items)

**String TCP ports**. Convert `items` to _str_. Parameter `items` can be as _str_, as list of _str_,
as list of _int_. Parameter `items` can be unsorted and with duplicates. Output data will be ordered
and without duplicates. Raise ValueError if ports are outside valid TCP/UDP ports 1...65535.

```python
import netports

# Convert <str> to sorted <str>. Output ordered and without duplicates.
assert netports.stcp("1,3-5,1,3-5") == "1,3-5"
# Raise ValueError if items are outside of range 1...65535.
try:
    netports.stcp("65536")
except ValueError:
    print("invalid tcp port")
```

## netports.ivlan(items)

**Integer VLAN IDs**. Convert `items` to list of _int_. Parameter `items` can be as _str_, as list
of _str_, as list of _int_. Parameter `items` can be unsorted and with duplicates. Output data will
be ordered and without duplicates. Raise ValueError if vlans are outside valid VLAN IDs 1...4094.

```python
import netports

# Convert <str> to list of <int>. Output ordered and without duplicates. Cisco style.
assert netports.ivlan("1,3-5,1,3-5") == [1, 3, 4, 5]
# Convert Hewlett Packard Enterprise style.
assert netports.ivlan("1 3 to 5", splitter=" ", range_splitter=" to ") == [1, 3, 4, 5]
# Raise ValueError if items are outside of range 1...4094.
try:
    netports.ivlan("4095")
except ValueError:
    print("invalid vlan port")
```

## netports.ivlan_hpe(items)

**Integer VLAN IDs, Hewlett Packard Enterprise style**. Convert `items` to list of _int_.
Parameter `items` can be as _str_, as list of _str_, as list of _int_. Parameter `items` can be
unsorted and with duplicates. Output data will be ordered and without duplicates. Raise ValueError
if vlans are outside valid VLAN IDs 1...4094.

```python
import netports

# Convert <str> to list of <int>. Output ordered and without duplicates. Cisco style.
assert netports.ivlan_hpe("1 3 to 5") == [1, 3, 4, 5]
# Raise ValueError if items are outside of range 1...4094.
try:
    netports.ivlan_hpe("4095")
except ValueError:
    print("invalid vlan port")
```

## netports.svlan(items)

**String VLAN IDs**. Convert `items` to _str_. Parameter `items` can be as _str_, as list of _str_,
as list of _int_. Parameter `items` can be unsorted and with duplicates. Output data will be ordered
and without duplicates. Raise ValueError if vlans are outside valid VLAN IDs 1...4094.

```python
import netports

# Convert <str> to sorted <str>. Output ordered and without duplicates. Cisco style.
assert netports.svlan("1,3-5,1,3-5") == "1,3-5"
# Convert Hewlett Packard Enterprise style.
assert netports.ivlan("1 3 to 5 1 3 to 5", splitter=" ", range_splitter=" to ") == [1, 3, 4, 5]
# Raise ValueError if items are outside of range 1...4094.
try:
    netports.svlan("4095")
except ValueError:
    print("invalid vlan port")
```

## netports.svlan_hpe(items)

**String VLAN IDs, Hewlett Packard Enterprise style**. Convert `items` to _str_. Parameter `items`
can be as _str_, as list of _str_, as list of _int_. Parameter `items` can be unsorted and with
duplicates. Output data will be ordered and without duplicates. Raise ValueError if vlans are
outside valid VLAN IDs 1...4094.

```python
import netports

# Convert <str> to sorted <str>. Output ordered and without duplicates. Cisco style.
assert netports.svlan("1,3-5,1,3-5") == "1,3-5"
# Convert Hewlett Packard Enterprise style.
assert netports.ivlan("1 3 to 5 1 3 to 5", splitter=" ", range_splitter=" to ") == [1, 3, 4, 5]
# Raise ValueError if items are outside of range 1...4094.
try:
    netports.svlan("4095")
except ValueError:
    print("invalid vlan port")
```

## netports.Interface4(line)

**Interface** - object that can contain up to 4 indexes. Useful for sorting interface strings by
index (not by alphabetic).

```python
from netports import Interface4

lines = [
    "interface Eth1/1/1.1",
    "interface Eth10/1/1.1",
    "interface Eth2/1/1.1",
    "interface Eth1/2/1.1",
    "interface Eth1/20/1.1",
    "interface Eth1/3/1.1",
]
# Example showing what alphabetical sorting looks like
for line in sorted(lines):
    print(line)
print()
# interface Eth1/1/1.1
# interface Eth1/2/1.1
# interface Eth1/20/1.1
# interface Eth1/3/1.1
# interface Eth10/1/1.1
# interface Eth2/1/1.1

# Sorting by indexes.
interfaces = [Interface4(line) for line in lines]
for interface in sorted(interfaces):
    print(interface)
print()
# interface Eth1/1/1.1
# interface Eth1/2/1.1
# interface Eth1/3/1.1
# interface Eth1/20/1.1
# interface Eth2/1/1.1
# interface Eth10/1/1.1
```