"""Interface that has up to 4 indexes"""
import re
from functools import total_ordering

from netports.types_ import TStr5


@total_ordering
class Interface4:
    """Interface that has up to 4 indexes"""

    def __init__(self, line: str = None):
        """Interface that has up to 4 indexes.
        :param line: Interface line.
        """
        self.line = line or ""

    def __str__(self):
        return self.line

    def __repr__(self):
        return f"{self.__class__.__name__}({self.line!r})"

    def __hash__(self) -> int:
        return hash((self.name, self.id1, self.id2, self.id3, self.id4))

    def __eq__(self, other) -> bool:
        """== equality"""
        if self.__class__ == other.__class__:
            if self.__hash__() == other.__hash__():
                return True
        return False

    def __lt__(self, other) -> bool:
        """< less than"""
        if self.__class__ == other.__class__:
            if self.name != other.name:
                return self.name < other.name
            if self.id1 != other.id1:
                return self.id1 < other.id1
            if self.id2 != other.id2:
                return self.id2 < other.id2
            if self.id3 != other.id3:
                return self.id3 < other.id3
            return self.id4 < other.id4
        return False

    # =========================== property ===========================

    @property
    def line(self) -> str:
        """Interface line"""
        return self._line

    @line.setter
    def line(self, line: str) -> None:
        if not isinstance(line, str):
            raise TypeError(f"{line=} {str} expected")
        items: TStr5 = self._parse_intf(line)
        self._line = line
        self._name = items[0]
        self._id1 = int(items[1]) if items[1] else 0
        self._id2 = int(items[2]) if items[2] else 0
        self._id3 = int(items[3]) if items[3] else 0
        self._id4 = int(items[4]) if items[4] else 0

    @property
    def name(self) -> str:
        """Interface name"""
        return self._name

    @property
    def id1(self) -> int:
        """Interface 1st index"""
        return self._id1

    @property
    def id2(self) -> int:
        """Interface 2nd index"""
        return self._id2

    @property
    def id3(self) -> int:
        """Interface 3rd index"""
        return self._id3

    @property
    def id4(self) -> int:
        """Interface 4th index"""
        return self._id4

    # =========================== helpers ============================

    @staticmethod
    def _parse_intf(intf: str) -> TStr5:
        """Parse interface to name and indexes.
        :param intf: Interface that has up to 4 indexes as <str>.
        :return: split name and indexes
        Example:
            intf: "interface Ethernet1/2/3.4"
            return: ("interface Ethernet", 1, 2, 3, 4)
        """
        delimiter = r"[,./:]"
        intf_ = re.sub(delimiter, ",", intf)
        name = r"([a-zA-Z\-\s]+)*"
        id1 = r"(\d+)*"
        id2 = r"(?:,)*(\d+)*"
        pattern = f"{name}{id1}{id2}{id2}{id2}"
        parsed: TStr5 = (re.findall(pattern, intf_) or [("", "", "", "", "")])[0]
        return parsed
