"""VLAN, TCP ports functions"""

import re
from typing import Any

from netports import helpers as h
from netports.ranges import Ranges, check_vlans_range, check_tcp_range
from netports.static import RANGE_SPLITTER, RANGE_SPLITTER_HPE, SPLITTER, SPLITTER_HPE
from netports.types_ import LStr, LInt


# ============================== ports ===============================

def ranges(line: str, **kwargs) -> Ranges:
    """Return Ranges <object>. Convert <str> to <object>.
    :param line: Input data syntax should be accurate, but can contain duplicates.
    :param kwargs: Optional parameters.
        splitter: Delimiter between items. By default ",".
        range_splitter: Delimiter between min and max digits in range. By default "-".
    :return: Output will be ordered and without duplicates.

    Example: Convert <str> to <object> and remove duplicates.
        param line: "3-5,1,3-5"
        return: Ranges("1,3-5")
    """
    splitter = str(kwargs.get("splitter") or SPLITTER)
    range_splitter = str(kwargs.get("range_splitter") or RANGE_SPLITTER)
    regex = r"(\s+)?{}(\s+)?".format(range_splitter)
    items_: LStr = h.list_of_str(line)
    items_ = [re.sub(regex, range_splitter, s) for s in items_]
    items_ = [splitter.join(s.split()) for s in items_ if s]
    items_ = [s for s in items_ if s]
    line = splitter.join(items_)
    ranges_ = Ranges(line, **kwargs)
    return ranges_


def iports(items: Any, **kwargs) -> LInt:
    """Integer Ports. Convert <str> or <list> to sorted list of <int>.
    :param items: Can be as <str>, as list of <str>, as list of <int>.
        Input data can be unsorted and with duplicates.
    :param kwargs: Optional parameters.
        splitter: Delimiter between items. By default ",".
        range_splitter: Delimiter between min and max digits in range. By default "-".
    :return: Output will be ordered and without duplicates.

    Example1: Convert list of <int> to list of <int> and remove duplicates.
        param items: [5, 5, 3, 4, 1]
        return: [1, 3, 4, 5]

    Example2: Convert list of <int> to list of <int> and remove duplicates.
        param items: "1, 7-9, 3 - 5"
        return: [1, 3, 4, 5, 7, 8, 9]
    """
    ranges_: Ranges = ranges(items, **kwargs)
    return ranges_.ports


def sports(items: Any, **kwargs) -> str:
    """String Ports. Convert <str> or <list> to sorted <str>.
    :param items: Can be as <str>, as list of <str>, as list of <int>.
        Input data can be unsorted and with duplicates.
    :param kwargs: Optional parameters.
        splitter: Delimiter between items. By default ",".
        range_splitter: Delimiter between min and max digits in range. By default "-".
    :return: Output will be ordered and without duplicates.

    Example1: Convert list of <int> to <str> and remove duplicates.
        :param items: [5, 5, 3, 4, 1]
        :return: "1,3-5"

    Example2: Convert <str> to <str> and remove duplicates.
        :param items: "1, 7-9, 3 - 5"
        :return: "1,3-5,7-9"
    """
    ranges_: Ranges = ranges(items, **kwargs)
    return str(ranges_)


# =============================== tcp ================================

def itcp(items: Any) -> LInt:
    """Integer TCP ports. Convert <str> or <list> to sorted list of <int>.
    Raise ValueError if items are outside of range 1...65535.
    :param items: Can be as <str>, as list of <str>, as list of <int>.
        Input data can be unsorted and with duplicates.
    :return: Output will be ordered and without duplicates.
    """
    ports: LInt = iports(items)
    check_tcp_range(ports=ports)
    return ports


def stcp(items: Any) -> str:
    """String TCP ports. Convert <str> or <list> to sorted <str>.
    Raise ValueError if items are outside of range 1...65535.
    :param items: Can be as <str>, as list of <str>, as list of <int>.
        Input data can be unsorted and with duplicates.
    :return: Output will be ordered and without duplicates.
    """
    ranges_: Ranges = ranges(items)
    check_tcp_range(ports=ranges_.ports)
    return str(ranges_)


# =============================== vlan ===============================

def ivlan(items: Any, **kwargs) -> LInt:
    """Integer VLAN IDs. Cisco style, bu default.
    Convert <str> or <list> to sorted list of <int> in range 1...4094.
    :param items: Can be as <str>, as list of <str>, as list of <int>.
        Input data can be unsorted and with duplicates.
    :param kwargs: Optional parameters.
        splitter: Delimiter between items. By default ",".
        range_splitter: Delimiter between min and max digits in range. By default "-".
    :return: Output will be ordered and without duplicates.

    Example1: Cisco style.
        items: "1,3-5"
        return: [1, 3, 4, 5]

    Example2: Hewlett Packard Enterprise style.
        items: "1 3 to 5"
        splitter: " "
        range_splitter: " to "
        return: [1, 3, 4, 5]
    """
    vlans: LInt = iports(items, **kwargs)
    check_vlans_range(vlans=vlans)
    return vlans


def ivlan_hpe(items: Any) -> LInt:
    """Integer VLAN IDs. Hewlett Packard Enterprise style.
    Convert <str> or <list> to sorted list of <int> in range 1...4094.
    :param items: Can be as <str>, as list of <str>, as list of <int>.
        Input data can be unsorted and with duplicates.
    :return: Output will be ordered and without duplicates.

    Example: Hewlett Packard Enterprise style.
        items: "1 3 to 5"
        return: [1, 3, 4, 5]
    """
    vlans: LInt = iports(items, splitter=SPLITTER_HPE, range_splitter=RANGE_SPLITTER_HPE)
    check_vlans_range(vlans=vlans)
    return vlans


def svlan(items: Any, **kwargs) -> str:
    """String VLAN IDs. Cisco style, bu default.
    Convert <str> or <list> to sorted <str> in range 1...4094.
    :param items: Can be as <str>, as list of <str>, as list of <int>.
        Input data can be unsorted and with duplicates.
    :param kwargs: Optional parameters.
        splitter: Delimiter between items. By default ",".
        range_splitter: Delimiter between min and max digits in range. By default "-".
    **kwargs
    :return: Output will be ordered and without duplicates.

    Example1: Cisco style.
        items: [1, 3, 4, 5]
        return: "1,3-5"

    Example2: Hewlett Packard Enterprise style.
        items: [1, 3, 4, 5]
        splitter: " "
        range_splitter: " to "
        return: "1 3 to 5"
    """
    ranges_: Ranges = ranges(items, **kwargs)
    check_vlans_range(vlans=ranges_.ports)
    return str(ranges_)


def svlan_hpe(items: Any) -> str:
    """String VLAN IDs. Hewlett Packard Enterprise style.
    Convert <str> or <list> to sorted <str> in range 1...4094.
    :param items: Can be as <str>, as list of <str>, as list of <int>.
        Input data can be unsorted and with duplicates.
    :return: Output will be ordered and without duplicates.

    Example: Hewlett Packard Enterprise style.
        items: [1, 3, 4, 5]
        return: "1 3 to 5"
    """
    ranges_: Ranges = ranges(items, splitter=SPLITTER_HPE, range_splitter=RANGE_SPLITTER_HPE)
    check_vlans_range(vlans=ranges_.ports)
    return str(ranges_)
